; Copyright (C) 2023  Val Lorentz
; 
; This program is free software: you can redistribute it and/or modify it under the
; terms of the GNU Affero General Public License version 3, as published by the Free
; Software Foundation.
; 
; This program is distributed in the hope that it will be useful, but WITHOUT ANY
; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
; PARTICULAR PURPOSE.  See the GNU Affero General Public License for more details.
; 
; You should have received a copy of the GNU Affero General Public License along with
; this program.  If not, see <http://www.gnu.org/licenses/>.

(ns jable.client
  (:require [clojure.tools.logging :refer :all]
            [clojure.java.io :as io]
            [clojure.string :as str]
            [jepsen [cli :as cli]
                    [client :as client]
                    [control :as c]
                    [db :as db]
                    [tests :as tests]]
            [jepsen.control.util :as cu]
            [jepsen.os.debian :as debian])
  (:import java.net.Socket))

(defn escape-tag-value [v]
  (str/replace v #"[; \\\r\n]" (fn [c]
                                 (case c
                                   ";" "\\:"
                                   " " "\\s"
                                   "\\" "\\\\"
                                   "\r" "\\r"
                                   "\n" "\\n"))))

(defn write-command [writer {tags :tags, :or {}
                             cmd :cmd
                             params :params}]
  (if (and tags (not (empty? tags)))
    (do
      (.write writer "@")
      (doseq [[i [k v]] (map-indexed vector tags)]
        (do
          (if (= i 0)
            nil
            (.write writer ";"))
          (.write writer k)
          (.write writer "=")
          (.write writer (escape-tag-value (or v "")))))
      (.write writer " "))
    nil)
  (.write writer cmd)
  (.write writer
    (if (empty? params)
      ""
      (let [first-params (pop params)
            trailing-param (last params)]
        (apply str (map #(str " " %) (conj first-params (str ":" trailing-param)))))))
  (.write writer "\r\n"))

(defn send-command [client command]
  (write-command (:writer client) command)
  (.flush (:writer client)))

(defn send-commands [client commands]
  (doseq [command commands]
    (write-command (:writer client) command))
  (.flush (:writer client)))

(defn unescape-tag-value [v]
  (str/replace v #"\\(.)" (fn [[_ c]]
                            (case c
                              ":" ";"
                              "s" " "
                              "\\" "\\"
                              "r" "\r"
                              "n" "\n"
                              c))))

(defn parse-tag [[k v]]
  (if v
    [k (unescape-tag-value v)]
    [k ""]))

(defn parse-tags [s]
  (as-> s v
    (str/split v #";")
    (map #(str/split % #"=" 2) v)
    (map parse-tag v)
    (into {} v)))

(def line-re
  #"^(?:@([^ ]+) )?(?::([^ ]+) )?([^ ]+)(?: (.*?))??(?: :(.*))?$")

(defn parse-command [line]
  (let [[_ tags source cmd params trailing] (re-matches line-re line)]
    {:tags (if tags (parse-tags tags) {})
     :source source
     :cmd cmd
     :params (let [params (if (empty? params)
                            []
                            (str/split params #" "))]
               (if trailing
                 (conj params trailing)
                 params))}))
